#include                <string.h>
#include                "tap.h"
#include                "libFireBird.h"

#define PROGRAM_NAME    "ChannelMatrix"
#define VERSION         "V0.1"
#define LOG             "ChannelMatrix.log"

TAP_ID                  (0x8E0A4256);
TAP_PROGRAM_NAME        (PROGRAM_NAME" "VERSION);
TAP_AUTHOR_NAME         ("FireBird");
TAP_DESCRIPTION         ("ChannelMatrix");
TAP_ETCINFO             (__DATE__);

typedef struct
{
  char                  SatName[MAX_SatName + 1];
  byte                  satIdx;
  bool                  Band;
  bool                  Polar;  //H=true
  dword                 Freq;
  char                  ChannelName[MAX_SvcName + 1 + 10];
  word                  OrigIndex;
  int                   SvcType;
} tChannelList;

typedef enum
{
  ST_DrawTVChannels,
  ST_WaitUserEntryTV,
  ST_WaitUserEntryRadio,
  ST_CreateReport,
  ST_Exit
} tState;

tChannelList            ChannelList[5000];
int                     ChannelListSize;
TYPE_File              *f;
int                     LastTVChannel, LastRadioChannel;
tState                  State;

void ShowTVChannelList(void)
{
  int                   nTvSvc, nRadioSvc, i;
  TYPE_TapChInfo        chInfo;

  TAP_Channel_GetTotalNum(&nTvSvc, &nRadioSvc);
  OSDMenuInitialize(FALSE, FALSE, TRUE, TRUE, "ChannelMatrix", "TV channels");
  for(i = 0; i < nTvSvc; i++)
  {
    TAP_Channel_GetInfo(SVC_TYPE_Tv, i, &chInfo);
    OSDMenuItemAdd(ValidFileName(chInfo.chName, ControlChars), NULL, NULL, NULL, TRUE, FALSE, 0);
  }

  OSDMenuSelectItem(0);
  OSDMenuButtonAdd(1, BI_Ok, NULL, "Select channel");
  OSDMenuButtonAdd(1, BI_Exit, NULL, "Exit TAP");

  OSDMenuUpdate(FALSE);
}

void ShowRadioChannelList(void)
{
  int                   nTvSvc, nRadioSvc, i;
  TYPE_TapChInfo        chInfo;

  TAP_Channel_GetTotalNum(&nTvSvc, &nRadioSvc);
  OSDMenuInitialize(FALSE, FALSE, TRUE, TRUE, "ChannelMatrix", "Radio channels");
  for(i = 0; i < nTvSvc; i++)
  {
    TAP_Channel_GetInfo(SVC_TYPE_Radio, i, &chInfo);
    OSDMenuItemAdd(ValidFileName(chInfo.chName, ControlChars), NULL, NULL, NULL, TRUE, FALSE, 0);
  }

  OSDMenuSelectItem(0);
  OSDMenuButtonAdd(1, BI_Ok, NULL, "Select channel");
  OSDMenuButtonAdd(1, BI_Exit, NULL, "Exit TAP");

  OSDMenuUpdate(FALSE);
}

void SwapChannelList(tChannelList *cl1, tChannelList *cl2)
{
  tChannelList          TempCl;

  memcpy(&TempCl, cl1, sizeof(tChannelList));
  memcpy(cl1, cl2, sizeof(tChannelList));
  memcpy(cl2, &TempCl, sizeof(tChannelList));
}

void BuildChannelList(void)
{
  int                   i;
  TYPE_TapChInfo        chInfo;
  tChannelList         *cl;

  ChannelListSize = 0;

  for(i = 0; i < LastTVChannel; i++)
  {
    cl = &ChannelList[ChannelListSize];

    TAP_Channel_GetInfo(SVC_TYPE_Tv, i, &chInfo);

    strcpy(cl->SatName, chInfo.satName);
    cl->satIdx = chInfo.satIdx;
    cl->Band = chInfo.freq > 11600;
    cl->Polar = chInfo.polar;
    cl->Freq = chInfo.freq;
    strcpy(cl->ChannelName, chInfo.chName);
    MakeValidFileName(cl->ChannelName, ControlChars);
    cl->OrigIndex = i;
    cl->SvcType = SVC_TYPE_Tv;

    ChannelListSize++;
  }

  for(i = 0; i < LastRadioChannel; i++)
    {
      cl = &ChannelList[ChannelListSize];

      TAP_Channel_GetInfo(SVC_TYPE_Radio, i, &chInfo);

      strcpy(cl->SatName, chInfo.satName);
      cl->satIdx = chInfo.satIdx;
      cl->Band = chInfo.freq > 11600;
      cl->Polar = chInfo.polar;
      cl->Freq = chInfo.freq;
      strcpy(cl->ChannelName, chInfo.chName);
      MakeValidFileName(cl->ChannelName, ControlChars);
      cl->OrigIndex = i;
      cl->SvcType = SVC_TYPE_Radio;

      ChannelListSize++;
    }
}

void SortChannelList(void)
{
  int                   i, j;
  tChannelList         *cl1, *cl2;

  for(i = 0; i < ChannelListSize - 1; i++)
  {
    cl1 = &ChannelList[i];
    for(j = i + 1; j < ChannelListSize; j++)
    {
      cl2 = &ChannelList[j];

      if(strcasecmp(cl1->SatName, cl2->SatName) > 0)
        SwapChannelList(cl1, cl2);
      else if(strcmp(cl1->SatName, cl2->SatName) == 0)
      {
        if(cl1->Band > cl2->Band)
          SwapChannelList(cl1, cl2);
        else if(cl1->Band == cl2->Band)
        {
          if(cl1->Polar > cl2->Polar)
            SwapChannelList(cl1, cl2);
          else if(cl1->Polar == cl2->Polar)
          {
            if(cl1->Freq > cl2->Freq)
              SwapChannelList(cl1, cl2);
            else if(cl1->Freq == cl2->Freq)
            {
              if(strcasecmp(cl1->ChannelName, cl2->ChannelName) > 0)
                SwapChannelList(cl1, cl2);
            }
          }
        }
      }
    }
  }
}

void PrintSatName(tChannelList *cl)
{
  char                  s[80];

  TAP_SPrint(s, "* %s\n", cl->SatName);
  TAP_PrintNet(s);
  TAP_Hdd_Fwrite(s, 1, strlen(s), f);
}

void PrintBandAndPolar(tChannelList *cl)
{
  char                  s[80];

  TAP_SPrint(s, "  * %s Band / %s\n", cl->Band ? "High" : "Low", cl->Polar ? "Horizontal" : "Vertikal");
  TAP_PrintNet(s);
  TAP_Hdd_Fwrite(s, 1, strlen(s), f);
}

void PrintChannelList(void)
{
  char                  s[1024];
  tChannelList         *cl;
  int                   i;
  bool                  LastBand = FALSE, LastPolar = FALSE;
  dword                 LastFreq = 0;
  byte                  LastSatIdx = 100;

  TAP_Hdd_Delete(LOG);
  TAP_Hdd_Create(LOG, ATTR_NORMAL);
  f = TAP_Hdd_Fopen(LOG);

  s[0] = '\0';

  for(i = 0; i < ChannelListSize; i++)
  {
    cl = &ChannelList[i];

    if(cl->satIdx != LastSatIdx)
    {
      LastSatIdx = cl->satIdx;
      LastBand = cl->Band;
      LastPolar = cl->Polar;
      LastFreq = cl->Freq;

      if(s[0])
      {
        strcat(s, "\n\n");
        TAP_PrintNet(s);
        TAP_Hdd_Fwrite(s, 1, strlen(s), f);
      }
      PrintSatName(cl);
      PrintBandAndPolar(cl);
      TAP_SPrint(s, "    * %d: %s(%s%d)", cl->Freq, cl->ChannelName, cl->SvcType == SVC_TYPE_Tv ? "T" : "R", cl->OrigIndex + 1);
    }
    else if((cl->Band != LastBand) || (cl->Polar != LastPolar))
    {
      LastBand = cl->Band;
      LastPolar = cl->Polar;
      LastFreq = cl->Freq;

      strcat(s, "\n\n");
      TAP_PrintNet(s);
      TAP_Hdd_Fwrite(s, 1, strlen(s), f);
      PrintBandAndPolar(cl);
      TAP_SPrint(s, "    * %d: %s(%s%d)", cl->Freq, cl->ChannelName, cl->SvcType == SVC_TYPE_Tv ? "T" : "R", cl->OrigIndex + 1);
    }
    else if(cl->Freq != LastFreq)
    {
      LastFreq = cl->Freq;

      strcat(s, "\n");
      TAP_PrintNet(s);
      TAP_Hdd_Fwrite(s, 1, strlen(s), f);
      TAP_SPrint(s, "    * %d: %s(%s%d)", cl->Freq, cl->ChannelName, cl->SvcType == SVC_TYPE_Tv ? "T" : "R", cl->OrigIndex + 1);
    }
    else if(strlen(s) < 990)
      TAP_SPrint(&s[strlen(s)], ", %s(%s%d)", cl->ChannelName, cl->SvcType == SVC_TYPE_Tv ? "T" : "R", cl->OrigIndex + 1);
  }

  strcat(s, "\n");
  TAP_PrintNet(s);
  TAP_Hdd_Fwrite(s, 1, strlen(s), f);
  TAP_Hdd_Fclose(f);
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
  (void) event;
  (void) param2;

  switch(State)
  {
    case ST_DrawTVChannels:
    {
      ShowTVChannelList();
      State = ST_WaitUserEntryTV;
      break;
    }

    case ST_WaitUserEntryTV:
    {
      OSDMenuEvent(&event, &param1, &param2);

      if(event == EVT_KEY)
      {
        switch(param1)
        {
          case RKEY_Ok:
          {
            LastTVChannel = OSDMenuGetCurrentItem();
            LastRadioChannel = 0;
            OSDMenuDestroyNoOSDUpdate();
            ShowRadioChannelList();
            State = ST_WaitUserEntryRadio;
            break;
          }

          case RKEY_Exit:
          {
            OSDMenuDestroy();
            State = ST_Exit;
            break;
          }
        }
      }
      break;
    }

    case ST_WaitUserEntryRadio:
    {
      OSDMenuEvent(&event, &param1, &param2);

      if(event == EVT_KEY)
      {
        switch(param1)
        {
          case RKEY_Ok:
          {
            LastRadioChannel = OSDMenuGetCurrentItem();
            OSDMenuDestroy();
            State = ST_CreateReport;
            break;
          }

          case RKEY_Exit:
          {
            OSDMenuDestroy();
            State = ST_Exit;
            break;
          }
        }
      }
      break;
    }

    case ST_CreateReport:
    {
      BuildChannelList();
      SortChannelList();
      PrintChannelList();
      State = ST_Exit;
      break;
    }

    case ST_Exit:
    {
      TAP_Exit();
      break;
    }
  }

  return (event == EVT_KEY) ? 0 : param1;
}

int TAP_Main(void)
{
  State = ST_DrawTVChannels;
  return 1;
}
